/*************************************************************************
Title:    ad8361.c
Version:  0.1 / 10. Maerz 2003
Author:   Ingo Gerlach , DH1AAD , http://www.welfen-netz.home/IngoGerlach

Software: AVR-GCC 3.3
Hardware: AD ADC1001 und AD8361
		  HD44780 compatible LCD
          ATS90S8515

**************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include "../lib/lcd.h"

/* constant definitions */
// 4 / 1023 = 0.0048...wegen Rundung eine Stelle gestrichen



/*
t = (255-TC0_RELOAD)*TC0_DELAY*TEILER * T_cyl
t = (255-60)*  3  *5*1024*250nS = 0.75 sec = in 3 sec zwei mal Anzeige aktualisieren
*/

#define ADC1001			1	// Je nach eingesetztem Type 1 = ADC1001 , 0 = ADC0803 (8Bit)

#if ADC1001 == 1
 #define ADC_LIMIT	1000	// Anzeige von * im Display bei erreichen des Limits
#else
  #define ADC_LIMIT	245
#endif

#if ADC1001 == 1
 #define ADC_REF	0.048875
#else
 #define ADC_REF	0.196078
#endif

#define TC0_DELAY 		3
#define TC0_RELOAD		60

#define ADC_PORT 	 PORTC	// Port Data
#define ADC_PINS	 PINC	// Port "        "
#define ADC_DDR		 DDRC	// data direction register for port

#define ADC_C_PORT 	 PORTB	// Port Control
#define ADC_C_PINS	 PINB	// Port "        "
#define ADC_C_DDR	 DDRB	// data direction register for port
#define ADC_C_RD	 0		// Pin 0 , RD
#define ADC_C_CS	 1		// Pin 1 , CS
#define ADC_C_INTR	 2		// Pin 2 , INTR
#define ADC_C_WRITE	 3		// Pin 3 , WRITE
#define ATT_PIN 	 7		// Pin 7 , switch

#ifndef UCSRB
# ifdef UCSR1A          /* ATmega128 */
#  define UCSRA UCSR1A
#  define UCSRB UCSR1B
#  define UBRR UBRR1L
#  define UDR UDR1
# else /* ATmega8 */
#  define UCSRA USR
#  define UCSRB UCR
# endif
#endif

#define SYSCLK 4000000UL      /* CPU clock in Hertz */

/* ------------------------------------------------------------*/

/* Type Def's */
typedef unsigned char  u08;
typedef          char  s08;
typedef unsigned short u16;
typedef          short s16;

/* ------------------------------------------------------------*/

/* Globals  */
u08 tc0_cnt = 0 ;			// Counter for Timer Delay
int adc_val = 0 ; 			// Calc ADC Value
char line1[20];
char line2[20];
long dbm;
// int avg;
long result_buf[2];    						    /* result buffer */
u16 att_buf[6] = {100,141,178,316,1000,3162};   /* Att Values 	 */
s08 att_arr[6] = {0,-3,-5,-10,-20,-30};    		/* Att Display	 */
u08 att_act    = 0;							    /* Actual Att 	 */
FILE *y;

/* ------------------------------------------------------------*/

/* function prototypes */

/* void wait_until_key_pressed(void); */
void delay_s(u08);
void read_adc();
void init_adc();
void calc_adc();
void check_att();
int uart_putchar(char c);
void ioinit(void);

/* ------------------------------------------------------------*/


/* ------------------------------------------------------------*/

SIGNAL(SIG_OVERFLOW0)     		/* signal handler for tcnt0 overflow interrupt */
{
  cli();
  tc0_cnt++;
  if (tc0_cnt > TC0_DELAY)
  {
    cbi(ADC_C_PORT,5);		// Clear auf high

	check_att();
	read_adc();					/* read ADC  0..1023 , bzw. 0.255 */
	calc_adc();
	// lcd_clrscr();

    if (adc_val > ADC_LIMIT) {
	 sprintf(line1,"* %5.3f Vrms %3d dB",(result_buf[1]/10000.0),att_arr[att_act]);
	} else { sprintf(line1,"%6.3f Vrms %4d dB",(result_buf[1]/10000.0),att_arr[att_act]);}
	if (att_act > 3 )
	{
	  sprintf(line2,"%6.3f W  %6.2fdBm",(result_buf[0]/1000000.0),(dbm/100.0));
    } else
      {
	    sprintf(line2,"%6.3f mW %6.2fdBm",(result_buf[0]/1000.0),(dbm/100.0));
      }
    lcd_gotoxy(0,0);
    lcd_puts(line1);
    lcd_gotoxy(0,1);
    lcd_puts(line2);
	tc0_cnt = 0;
	// Output to serial
	// sprintf(line1,"Akt %5d",adc_val);
	fprintf(y, ">%5d;%1d<\n",adc_val,att_act);
    sbi(ADC_C_PORT,5); 	// Output Enable auf low = enable
  }

  outp(TC0_RELOAD, TCNT0);         /* reset counter to get this interrupt again */
  sei();
}

void check_att()
{
  if (bit_is_clear(PIND,ATT_PIN)) att_act++;
  if(att_act > 5) { att_act = 0; }
}

void calc_adc()
{
// avg = (adc_val + avg ) / 2;
// result_buf[1] = (((avg*ADC_REF)/7.5)*10)*(att_buf[att_act]); // *1000
// result_buf[0] = ((result_buf[1]/10)*(result_buf[1]/10))/50.0;

 result_buf[1] = (((adc_val*ADC_REF)/7.5)*10)*(att_buf[att_act]); // *1000
 result_buf[0] = ((result_buf[1]/10)*(result_buf[1]/10))/50.0;

 dbm  = log10(((result_buf[0]/10000.0) / 0.1))*1000;
 if(dbm < -6000) { dbm = -6000.00; }
}

void read_adc()
{
 u08 adc_msb=0;
 u08 adc_lsb=0;

  cbi(ADC_C_PORT,ADC_C_CS); 	// Output Enable auf low = enable
  cbi(ADC_C_PORT,ADC_C_WRITE);	// Clear auf low
  cbi(ADC_C_PORT,ADC_C_RD); 	// Output Enable auf low = enable
  asm volatile ("nop\n"::);
  sbi(ADC_C_PORT,ADC_C_WRITE);	// Clear auf low
  asm volatile ("nop\n"::);
  asm volatile ("nop\n"::);

  adc_msb=inp(ADC_PINS);		// read ADC MSB
  asm volatile ("nop\n"::);
  asm volatile ("nop\n"::);

  #if ADC1001 == 1

  sbi(ADC_C_PORT,ADC_C_RD);		// Clear auf high
  asm volatile ("nop\n"::);

  cbi(ADC_C_PORT,ADC_C_RD); 	// Output Enable auf low = enable
  asm volatile ("nop\n"::);
  asm volatile ("nop\n"::);

  adc_lsb=inp(ADC_PINS);		// read ADC LSB

  sbi(ADC_C_PORT,ADC_C_RD);		// Clear auf high
  sbi(ADC_C_PORT,ADC_C_CS);		// Clear auf high

  adc_val = (adc_msb * 4) + (adc_lsb / 64);
 #else
  sbi(ADC_C_PORT,ADC_C_RD);		// Clear auf high
  sbi(ADC_C_PORT,ADC_C_CS);		// Clear auf high
  adc_val = adc_msb;			// ADC0803 nur MSB
 #endif
}

void init_adc()
{
  outp(0x00,ADC_DDR);         	/* Input Port  */
  outp(0x3f,ADC_C_DDR);       	/* Control Port output, PIN 6,7 as input*/

  sbi(ADC_C_PORT,ADC_C_CS);	  	// CS auf high
  sbi(ADC_C_PORT,ADC_C_WRITE);	// Write auf high
  sbi(ADC_C_PORT,ADC_C_RD);	  	// Read auf high
  asm volatile ("nop\n"::);
  cbi(ADC_C_PORT,ADC_C_CS);   	// CS auf low
  asm volatile ("nop\n"::);
  cbi(ADC_C_PORT,ADC_C_WRITE);	// WR auf low
  asm volatile ("nop\n"::);
  asm volatile ("nop\n"::);
  sbi(ADC_C_PORT,ADC_C_CS);   	// CS auf high
  sbi(ADC_C_PORT,ADC_C_WRITE);	// WR auf high
}

/*
 * Send character c down the UART Tx, wait until tx holding register
 * is empty.
 */
int uart_putchar(char c)
{

  if (c == '\n')
   uart_putchar('\r');
  loop_until_bit_is_set(UCSRA, UDRE);
  outb(UDR, c);
  return 0;
}

void ioinit(void)
{
  UCSRB = _BV(TXEN);            /* tx enable */
  UBRR = (SYSCLK / (16 * 9600UL)) - 1; /* 9600 Bd */
}

/* To Do bergabe in Sekunden */
/*                       1	  */
/* delay (1 Sek.) = --------- */
/*                     65000  */
/*                    --------*/
/*                     4 MHz  */
void delay_s(u08 delay)
{
  int n;
  while(delay--) {
   n=65000;
   while(n--) { asm volatile ("nop\n"::); }
  }
}

int main(void)
{
  ioinit();
  y = fdevopen(uart_putchar, NULL, 0);
  fprintf(y, "Init AD8361...\n");

  lcd_init(LCD_DISP_ON);		// Display on
  lcd_led(1);					// Display Light on
  lcd_gotoxy(0,0);
  /*        01234567890ABCDEF */
  lcd_puts("(c) DH1AAD/03/03");
  lcd_gotoxy(0,1);
  lcd_puts("AD8361mW Init...");

  init_adc();
  delay_s(24); /* every second */
  lcd_clrscr();

  outp((1<<TOIE0), TIMSK); /* enable TCNT0 overflow */
  outp(TC0_RELOAD, TCNT0); /* reset TCNT0 */
  outp(5,   TCCR0);        /* count with cpu clock/1024 */

  // cbi(DDRD,PD2);		   /* Pin PD2 input */
  // sbi(PORTD,PD2);	   /* Pin PD2 pull-up enabled */

  sei();                   /* enable interrupts */
  for (;;) {}              /* loop forever */
}
